"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("../task/taskId");
const generateTask_1 = require("../task/generateTask");
exports.ComputeHashTask = "??computeHash";
exports.CacheFetchTask = "??fetch";
exports.CachePutTask = "??put";
function isCacheTask(task) {
    return (task === exports.ComputeHashTask || task === exports.CacheFetchTask || task === exports.CachePutTask);
}
exports.isCacheTask = isCacheTask;
function generateCacheTasks(context) {
    const { tasks, taskDepsGraph, cache } = context;
    if (context.cache) {
        for (const taskId of tasks.keys()) {
            const [pkg, task] = taskId_1.getPackageTaskFromId(taskId);
            if (task !== exports.CacheFetchTask &&
                task !== exports.CachePutTask &&
                task !== exports.ComputeHashTask &&
                pkg) {
                const hashTaskId = taskId_1.getTaskId(pkg, exports.ComputeHashTask);
                const fetchTaskId = taskId_1.getTaskId(pkg, exports.CacheFetchTask);
                const putTaskId = taskId_1.getTaskId(pkg, exports.CachePutTask);
                // set up the graph
                taskDepsGraph.push([hashTaskId, fetchTaskId]);
                tasks.set(hashTaskId, () => generateTask_1.generateTask(hashTaskId, context));
                taskDepsGraph.push([fetchTaskId, taskId]);
                tasks.set(fetchTaskId, () => generateTask_1.generateTask(fetchTaskId, context));
                taskDepsGraph.push([taskId, putTaskId]);
                tasks.set(putTaskId, () => generateTask_1.generateTask(putTaskId, context));
            }
        }
    }
}
exports.generateCacheTasks = generateCacheTasks;
