"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * identify and create a realized task dependency map (discovering)
 */
function discoverTaskDeps(allPackages, options) {
    const { defaultPipeline } = options;
    for (const [pkg, info] of Object.entries(allPackages)) {
        const pipeline = info.pipeline || defaultPipeline;
        for (const [task, taskDeps] of Object.entries(pipeline)) {
            if (info.scripts[task]) {
                const taskId = getTaskId(pkg, task);
                if (!tasksDepsMap.has(taskId)) {
                    tasksDepsMap.set(taskId, []);
                }
                for (const taskDep of taskDeps) {
                    if (taskDep.startsWith("^")) {
                        // add task dep from all the package deps within repo
                        const dependentPkgs = getInternalDepsWithTask(info, task, allPackages);
                        const taskName = taskDep.slice(1);
                        for (const depPkg of dependentPkgs) {
                            tasksDepsMap.get(taskId).push(getTaskId(depPkg, taskName));
                        }
                    }
                    else if (info.scripts[taskDep]) {
                        // add task dep from same package
                        tasksDepsMap.get(taskId).push(getTaskId(pkg, taskDep));
                    }
                }
            }
        }
    }
}
exports.discoverTaskDeps = discoverTaskDeps;
