"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const workspace_tools_1 = require("workspace-tools");
const discoverTaskDeps_1 = require("./task/discoverTaskDeps");
const taskRunner_1 = require("./task/taskRunner");
const profiler_1 = __importDefault(require("@lerna/profiler"));
const os_1 = __importDefault(require("os"));
const dist_1 = __importDefault(require("p-queue/dist"));
const cosmiconfig_1 = require("cosmiconfig");
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const events_1 = require("events");
const npmlog_1 = __importDefault(require("npmlog"));
const parsedArgs = yargs_parser_1.default(process.argv.slice(2));
const root = workspace_tools_1.findGitRoot(process.cwd());
if (!root) {
    throw new Error("This must be called inside a git-controlled repo");
}
const ConfigModuleName = "lage";
const configResults = cosmiconfig_1.cosmiconfigSync(ConfigModuleName).search(root || process.cwd());
const concurrency = os_1.default.cpus().length - 1;
const command = parsedArgs._[0];
const context = {
    allPackages: workspace_tools_1.getPackageInfos(root),
    command,
    concurrency,
    defaultPipeline: (configResults === null || configResults === void 0 ? void 0 : configResults.config.pipeline) || {
        build: ["^build"],
        clean: [],
    },
    taskDepsGraph: [],
    tasks: new Map(),
    deps: parsedArgs.deps || (configResults === null || configResults === void 0 ? void 0 : configResults.config.deps) || false,
    scope: parsedArgs.scope || (configResults === null || configResults === void 0 ? void 0 : configResults.config.scope) || [],
    measures: {
        start: [0, 0],
        duration: [0, 0],
        taskStats: [],
        failedTask: undefined,
    },
    profiler: new profiler_1.default({
        concurrency,
        outputDirectory: process.cwd(),
    }),
    taskLogs: new Map(),
    queue: new dist_1.default({ concurrency }),
    cache: parsedArgs.cache === false ? false : true,
    nodeArgs: parsedArgs.nodeArgs ? arrifyArgs(parsedArgs.nodeArgs) : [],
    args: getPassThroughArgs(parsedArgs),
    events: new events_1.EventEmitter(),
    verbose: parsedArgs.verbose ? true : false,
};
if (context.verbose) {
    npmlog_1.default.level = "verbose";
}
discoverTaskDeps_1.discoverTaskDeps(context);
taskRunner_1.runTasks(context);
function arrifyArgs(args) {
    const argsArray = [];
    for (const [key, val] of Object.entries(args)) {
        if (Array.isArray(val)) {
            for (const item of val) {
                pushValue(key, item);
            }
        }
        else {
            pushValue(key, val);
        }
    }
    return argsArray;
    function pushValue(key, value) {
        let keyArg = "";
        if (typeof value === "boolean") {
            if (key.length === 1 && value) {
                keyArg = `-${key}`;
            }
            else if (value) {
                keyArg = `--${key}`;
            }
            else {
                keyArg = `--no-${key}`;
            }
            argsArray.push(keyArg);
        }
        else {
            argsArray.push(keyArg, value);
        }
    }
}
function getPassThroughArgs(args) {
    let result = [];
    result = result.concat(args._.slice(1));
    let { nodeArgs: _nodeArgValues, scope: _scopeArg, deps: _depsArg, cache: _cacheArg, _: _positionals } = args, filtered = __rest(args, ["nodeArgs", "scope", "deps", "cache", "_"]);
    result = result.concat(arrifyArgs(filtered));
    return result;
}
