"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const paths_1 = require("../paths");
const git_1 = require("../git");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const minimatch_1 = __importDefault(require("minimatch"));
const read_yaml_1 = __importDefault(require("read-yaml"));
const jju_1 = __importDefault(require("jju"));
/**
 * Finds the git root, then either process yarn, pnpm workspaces minimatches vs what is checked into git repo
 * @param cwd
 */
function getAllPackageJsonFiles(cwd) {
    const gitRoot = paths_1.findGitRoot(cwd);
    // Rush handling
    if (fs_1.default.existsSync(path_1.default.join(gitRoot, "rush.json"))) {
        const rushConfig = jju_1.default.parse(fs_1.default.readFileSync(path_1.default.join(gitRoot, "rush.json"), "utf-8"));
        return rushConfig.projects.map((project) => path_1.default.join(project.projectFolder, "package.json"));
    }
    // get all tracked package.json's except the root one (assuming it is a workspace package.json)
    let packageJsonFiles = git_1.listAllTrackedFiles(["**/package.json"], gitRoot);
    if (!fs_1.default.existsSync(path_1.default.join(gitRoot, "package.json"))) {
        throw new Error("This tool is meant to be run in a workspace at the git root");
    }
    let packagePatterns = [];
    try {
        const packageJson = JSON.parse(fs_1.default.readFileSync(path_1.default.join(gitRoot, "package.json"), "utf-8"));
        if (packageJson.workspaces) {
            const workspaces = packageJson.workspaces;
            // read package.json workspace settings
            if (Array.isArray(workspaces)) {
                packagePatterns = workspaces;
            }
            else if (workspaces.packages) {
                packagePatterns = workspaces.packages;
            }
        }
    }
    catch (_a) {
        throw new Error("The package.json at the root is not formatted correctly");
    }
    // read pnpm-workspace.yaml if found
    if (fs_1.default.existsSync(path_1.default.join(gitRoot, "pnpm-workspace.yaml"))) {
        try {
            const workspaces = read_yaml_1.default.sync(path_1.default.join(gitRoot, "pnpm-workspace.yaml"));
            if (workspaces && workspaces.packages) {
                packagePatterns = workspaces.packages;
            }
        }
        catch (_b) {
            throw new Error("The pnpm-workspace.yaml is not formatted correctly");
        }
    }
    if (packagePatterns.length > 0) {
        // filter out the packageJsonFiles
        packageJsonFiles = packageJsonFiles.filter((packageJsonFile) => {
            let matched = false;
            for (const pattern of packagePatterns) {
                if (minimatch_1.default(path_1.default.dirname(packageJsonFile), pattern)) {
                    matched = true;
                    break;
                }
            }
            return matched;
        });
    }
    return packageJsonFiles;
}
exports.getAllPackageJsonFiles = getAllPackageJsonFiles;
