"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getTaskId(pkg, task) {
    return `${pkg}###${task}`;
}
exports.getTaskId = getTaskId;
function getPackageTaskFromId(id) {
    return id.split("###");
}
exports.getPackageTaskFromId = getPackageTaskFromId;
function generateTask(taskId, fn) {
    const taskDeps = tasksDepsMap.get(taskId);
    const prereqPromise = taskDeps
        ? Promise.all(taskDeps.map((d) => tasks.get(d)))
        : Promise.resolve();
    tasks.set(taskId, prereqPromise.then((_) => new Promise((resolve, reject) => {
        performance.mark(`start:${taskId}`);
        const [pkg, task] = getPackageTaskFromId(taskId);
        console.log(`----- Running ${pkg}: ${task} -----`);
        const results = fn(allPackages[pkg]);
        if (results instanceof Promise) {
            results.then(() => {
                console.log(`----- Done ${pkg}: ${task} -----`);
                resolve();
            }, (err) => {
                console.log(`----- FAILED ${pkg}: ${task} -----`);
                console.log(err);
                reject();
            });
        }
        else {
            console.log(`----- Done ${pkg}: ${task} -----`);
            resolve();
        }
    }), (err) => {
        const [pkg, task] = getPackageTaskFromId(taskId);
        console.log(`----- FAILED ${pkg}: ${task} -----`);
    }));
    return tasks.get(taskId);
}
exports.generateTask = generateTask;
