"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("./taskId");
const toposort_1 = __importDefault(require("toposort"));
/**
 * Using scoped entry points, generate the execution taskGraph (planning)
 * @param context
 */
function getSortedTaskIds(context) {
    const { command, taskDepsMap, allPackages } = context;
    // start taskStack with entry points (future: add scoped packages as well)
    const taskStack = [];
    for (const taskId of taskDepsMap.keys()) {
        const [_, taskName] = taskId_1.getPackageTaskFromId(taskId);
        if (taskName === command) {
            taskStack.push(taskId);
        }
    }
    for (const pkg of Object.keys(allPackages)) {
        taskStack.push(taskId_1.getTaskId(pkg, "_computeHash"));
        taskStack.push(taskId_1.getTaskId(pkg, "_fetch"));
        taskStack.push(taskId_1.getTaskId(pkg, "_put"));
    }
    const visited = new Set();
    const taskGraph = [];
    while (taskStack.length > 0) {
        const taskId = taskStack.pop();
        if (!visited.has(taskId)) {
            const deps = taskDepsMap.get(taskId);
            if (deps && deps.length > 0) {
                for (const depTaskId of deps) {
                    taskGraph.push([depTaskId, taskId]);
                    taskStack.push(depTaskId);
                }
            }
            else {
                taskGraph.push(["", taskId]);
            }
            visited.add(taskId);
        }
    }
    return toposort_1.default(taskGraph).filter((id) => id);
}
exports.getSortedTaskIds = getSortedTaskIds;
