"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("./taskId");
const generateTask_1 = require("./generateTask");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const backfill_1 = require("../cache/backfill");
const perf_hooks_1 = require("perf_hooks");
const ansiReporter_1 = require("../reporter/ansiReporter");
function generateNpmTask(taskId, context) {
    const [pkg, task] = taskId_1.getPackageTaskFromId(taskId);
    const { allPackages } = context;
    return generateTask_1.generateTask(taskId, () => new Promise((resolve, reject) => {
        if (backfill_1.cacheHits[pkg]) {
            return resolve();
        }
        const cp = child_process_1.spawn("npm.cmd", ["run", task], {
            cwd: path_1.default.dirname(allPackages[pkg].packageJsonPath),
            stdio: "pipe",
        });
        cp.stdout.on("data", (data) => {
            data
                .toString()
                .split(/\n/)
                .forEach((line) => {
                // if (line.trim()) {
                //   process.stdout.write(`${pkg}:${task}: ${line.trim()}\n`);
                // }
                ansiReporter_1.reportTaskLog(taskId, line, context);
            });
        });
        cp.stderr.on("data", (data) => {
            data
                .toString()
                .split(/\n/)
                .forEach((line) => {
                // if (line.trim()) {
                //   process.stderr.write(`${pkg}:${task}: ${line.trim()}\n`);
                // }
                ansiReporter_1.reportTaskLog(taskId, line, context);
            });
        });
        cp.on("exit", (code) => {
            perf_hooks_1.performance.mark(`end:${taskId}`);
            perf_hooks_1.performance.measure(`${taskId}`, `start:${taskId}`, `end:${taskId}`);
            if (code === 0) {
                return resolve();
            }
            reject();
        });
    }), context);
}
exports.generateNpmTask = generateNpmTask;
