"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("./taskId");
const cacheTasks_1 = require("../cache/cacheTasks");
const backfill_1 = require("../cache/backfill");
const npmTask_1 = require("./npmTask");
const taskWrapper_1 = require("./taskWrapper");
const EmptyTask = "";
/**
 * Create task wraps the queueing, returns the promise for completion of the task ultimately
 * @param taskId
 * @param context
 */
function generateTask(taskId, context) {
    const [_, task] = taskId_1.getPackageTaskFromId(taskId);
    if (taskId === EmptyTask) {
        return Promise.resolve();
    }
    switch (task) {
        case cacheTasks_1.ComputeHashTask:
            return taskWrapper_1.taskWrapper(taskId, backfill_1.computeHash, context);
        case cacheTasks_1.CacheFetchTask:
            return taskWrapper_1.taskWrapper(taskId, backfill_1.fetchBackfill, context);
        case cacheTasks_1.CachePutTask:
            return taskWrapper_1.taskWrapper(taskId, backfill_1.putBackfill, context);
        default:
            return npmTask_1.npmTask(taskId, context);
    }
}
exports.generateTask = generateTask;
