"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("./taskId");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const logger_1 = require("../logger");
const taskWrapper_1 = require("./taskWrapper");
const abortSignal_1 = require("./abortSignal");
function npmTask(taskId, context) {
    const [pkg, task] = taskId_1.getPackageTaskFromId(taskId);
    const { allPackages, queue } = context;
    const logger = logger_1.createLogger(taskId);
    const prefix = `${pkg} ${task}`;
    return queue.add(() => taskWrapper_1.taskWrapper(taskId, () => new Promise((resolve, reject) => {
        if (!allPackages[pkg].scripts[task]) {
            logger.info(prefix, `Empty script detected: ${pkg} - ${task}`);
            return resolve();
        }
        logger.verbose(prefix, `Running ${[
            process.execPath,
            ...context.nodeArgs,
            path_1.default.join(path_1.default.dirname(process.execPath), "node_modules/npm/bin/npm-cli.js"),
            "run",
            task,
            ...(context.args.length > 0 ? ["--", ...context.args] : []),
        ].join(" ")}`);
        const cp = child_process_1.spawn(process.execPath, [
            ...context.nodeArgs,
            path_1.default.join(path_1.default.dirname(process.execPath), "node_modules/npm/bin/npm-cli.js"),
            "run",
            task,
            "--",
            ...context.args,
        ], {
            cwd: path_1.default.dirname(allPackages[pkg].packageJsonPath),
            stdio: "pipe",
        });
        context.events.once("abort", () => {
            queue.pause();
            context.measures.failedTask = taskId;
            // kill the process in progress, but be gentle and let the process themselves take care of SIGTERM
            cp.kill("SIGTERM");
        });
        cp.stdout.on("data", (data) => {
            if (!cp.killed) {
                data
                    .toString()
                    .split(/\n/)
                    .forEach((line) => {
                    logger.verbose(prefix, line);
                });
            }
        });
        cp.stderr.on("data", (data) => {
            if (!cp.killed) {
                data
                    .toString()
                    .split(/\n/)
                    .forEach((line) => {
                    logger.verbose(prefix, line);
                });
            }
        });
        cp.on("exit", (code) => {
            if (code === 0) {
                return resolve();
            }
            abortSignal_1.abort(context);
            reject();
        });
    }), context));
}
exports.npmTask = npmTask;
