"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("./taskId");
const backfill_1 = require("../cache/backfill");
const logger_1 = require("../logger");
const cacheTasks_1 = require("../cache/cacheTasks");
async function taskWrapper(taskId, fn, context) {
    const { allPackages, profiler, measures, queue } = context;
    const [pkg, task] = taskId_1.getPackageTaskFromId(taskId);
    const prefix = `${pkg} ${task}`;
    if (!backfill_1.cacheHits[pkg]) {
        if (!cacheTasks_1.isCacheTask(task)) {
            logger_1.info(prefix, "started");
        }
        const start = process.hrtime();
        await profiler.run(() => fn(allPackages[pkg], context), taskId);
        if (!cacheTasks_1.isCacheTask(task)) {
            const duration = process.hrtime(start);
            measures.taskStats.push({
                taskId,
                start,
                duration,
            });
            logger_1.info(prefix, `ended - took ${(duration[0] + duration[1] / 1e9).toFixed(2)}s`);
        }
    }
    else if (!cacheTasks_1.isCacheTask(task)) {
        logger_1.info(prefix, "skipped");
    }
}
exports.taskWrapper = taskWrapper;
